#!/usr/bin/env python
import sys
import os

from django.conf import settings

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

settings.configure(
    SITE_ID=1,
    DATABASES={
        'default': {'ENGINE': 'django.db.backends.sqlite3', 'NAME': ':memory;'}
    },
    INSTALLED_APPS=[
        'django.contrib.auth',
        'django.contrib.sessions',
        'django.contrib.contenttypes',
        'django_social_share',
        'django_social_share.tests',
    ],
    TEMPLATES=[
        {
            'BACKEND': 'django.template.backends.django.DjangoTemplates',
            'DIRS': [
                os.path.join(BASE_DIR, 'templates'),
            ],
            'OPTIONS': {
                'context_processors': [
                    'django.template.context_processors.request',
                ],
            }
        },
    ]
)


def runtests(*test_args):
    import django
    try:
        django.setup()  # Django 1.7+
    except AttributeError:
        pass
    import django.test.utils
    runner_class = django.test.utils.get_runner(settings)
    test_runner = runner_class(verbosity=1, interactive=True)
    failures = test_runner.run_tests(['django_social_share'])
    sys.exit(failures)


if __name__ == '__main__':
    runtests()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import re

from django import template

from django.db.models import Model
from django.template.defaultfilters import urlencode
from django.utils.safestring import mark_safe

try:
    from django_bitly.templatetags.bitly import bitlify
    DJANGO_BITLY = True
except ImportError:
    DJANGO_BITLY = False


register = template.Library()


TWITTER_ENDPOINT = 'https://twitter.com/intent/tweet?text=%s'
FACEBOOK_ENDPOINT = 'https://www.facebook.com/sharer/sharer.php?u=%s'
GPLUS_ENDPOINT = 'https://plus.google.com/share?url=%s'
MAIL_ENDPOINT = 'mailto:?subject=%s&body=%s'
LINKEDIN_ENDPOINT = 'https://www.linkedin.com/shareArticle?mini=true&title=%s&url=%s'
REDDIT_ENDPOINT = 'https://www.reddit.com/submit?title=%s&url=%s'
TELEGRAM_ENDPOINT = 'https://t.me/share/url?text=%s&url=%s'
WHATSAPP_ENDPOINT = 'https://api.whatsapp.com/send?text=%s'
PINTEREST_ENDPOINT = 'https://www.pinterest.com/pin/create/button/?url=%s'


BITLY_REGEX = re.compile(r'^https?://bit\.ly/')


def compile_text(context, text):
    ctx = template.context.Context(context)
    return template.Template(text).render(ctx)


def _build_url(request, obj_or_url):
    if obj_or_url is not None:
        if isinstance(obj_or_url, Model):
            if DJANGO_BITLY:
                url = bitlify(obj_or_url)  # type: str
                if not BITLY_REGEX.match(url):
                    return request.build_absolute_uri(
                        obj_or_url.get_absolute_url()
                    )
                else:
                    return url
            else:
                return request.build_absolute_uri(obj_or_url.get_absolute_url())
        else:
            return request.build_absolute_uri(obj_or_url)
    return ''


def _compose_tweet(text, url=None):
    TWITTER_MAX_NUMBER_OF_CHARACTERS = 140
    TWITTER_LINK_LENGTH = 23  # "A URL of any length will be altered to 23 characters, even if the link itself is less than 23 characters long.

    # Compute length of the tweet
    url_length = len(' ') + TWITTER_LINK_LENGTH if url else 0
    total_length = len(text) + url_length

    # Check that the text respects the max number of characters for a tweet
    if total_length > TWITTER_MAX_NUMBER_OF_CHARACTERS:
        text = text[:(TWITTER_MAX_NUMBER_OF_CHARACTERS - url_length - 1)] + "…"  # len("…") == 1

    return "%s %s" % (text, url) if url else text


@register.simple_tag(takes_context=True)
def post_to_twitter_url(context, text, obj_or_url=None):
    text = compile_text(context, text)
    request = context['request']

    url = _build_url(request, obj_or_url)

    tweet = _compose_tweet(text, url)
    context['tweet_url'] = TWITTER_ENDPOINT % urlencode(tweet)
    return context


@register.inclusion_tag('django_social_share/templatetags/post_to_twitter.html', takes_context=True)
def post_to_twitter(context, text, obj_or_url=None, link_text='',link_class=""):
    context = post_to_twitter_url(context, text, obj_or_url)

    request = context['request']
    url = _build_url(request, obj_or_url)
    tweet = _compose_tweet(text, url)

    context['link_class'] = link_class 
    context['link_text'] = link_text or 'Post to Twitter'
    context['full_text'] = tweet
    return context


@register.simple_tag(takes_context=True)
def post_to_facebook_url(context, obj_or_url=None):
    request = context['request']
    url = _build_url(request, obj_or_url)
    context['facebook_url'] = FACEBOOK_ENDPOINT % urlencode(url)
    return context


@register.inclusion_tag('django_social_share/templatetags/post_to_facebook.html', takes_context=True)
def post_to_facebook(context, obj_or_url=None, link_text='',link_class=''):
    context = post_to_facebook_url(context, obj_or_url)
    context['link_class'] = link_class  or ''
    context['link_text'] = link_text or 'Post to Facebook'
    return context


@register.simple_tag(takes_context=True)
def post_to_gplus_url(context, obj_or_url=None):
    request = context['request']
    url = _build_url(request, obj_or_url)
    context['gplus_url'] = GPLUS_ENDPOINT % urlencode(url)
    return context


@register.inclusion_tag('django_social_share/templatetags/post_to_gplus.html', takes_context=True)
def post_to_gplus(context, obj_or_url=None, link_text='',link_class=''):
    context = post_to_gplus_url(context, obj_or_url)
    context['link_class'] = link_class 
    context['link_text'] = link_text or 'Post to Google+'
    return context


@register.simple_tag(takes_context=True)
def send_email_url(context, subject, text, obj_or_url=None):
    text = compile_text(context, text)
    subject = compile_text(context, subject)
    request = context['request']
    url = _build_url(request, obj_or_url)
    full_text = "%s %s" % (text, url)
    context['mailto_url'] = MAIL_ENDPOINT % (urlencode(subject), urlencode(full_text))
    return context


@register.inclusion_tag('django_social_share/templatetags/send_email.html', takes_context=True)
def send_email(context, subject, text, obj_or_url=None, link_text='',link_class=''):
    context = send_email_url(context, subject, text, obj_or_url)
    context['link_class'] = link_class 
    context['link_text'] = link_text or 'Share via email'
    return context


@register.filter(name='linkedin_locale')
def linkedin_locale(value):
    if "-" not in value:
        return value

    lang, country = value.split('-')
    return '_'.join([lang, country.upper()])


@register.simple_tag(takes_context=True)
def post_to_linkedin_url(context, obj_or_url=None):
    request = context['request']
    url = _build_url(request, obj_or_url)
    context['linkedin_url'] = url
    return context


@register.inclusion_tag('django_social_share/templatetags/post_to_linkedin.html', takes_context=True)
def post_to_linkedin(context, obj_or_url=None,link_class=''):
    context = post_to_linkedin_url(context, obj_or_url)
    context['link_class'] = link_class 
    return context


@register.simple_tag(takes_context=True)
def post_to_reddit_url(context, title, obj_or_url=None):
    request = context['request']
    title = compile_text(context, title)
    url = _build_url(request, obj_or_url)
    context['reddit_url'] = mark_safe(REDDIT_ENDPOINT % (urlencode(title), urlencode(url)))
    return context


@register.inclusion_tag('django_social_share/templatetags/post_to_reddit.html', takes_context=True)
def post_to_reddit(context, title, obj_or_url=None, link_text='',link_class=''):
    context = post_to_reddit_url(context, title, obj_or_url)
    context['link_class'] = link_class 
    context['link_text'] = link_text or 'Post to Reddit' 
    return context


@register.simple_tag(takes_context=True)
def post_to_telegram_url(context, title, obj_or_url):
    request = context['request']
    title = compile_text(context, title)
    url = _build_url(request, obj_or_url)
    context['telegram_url'] = mark_safe(TELEGRAM_ENDPOINT % (urlencode(title), urlencode(url)))
    return context


@register.inclusion_tag('django_social_share/templatetags/post_to_telegram.html', takes_context=True)
def post_to_telegram(context, title, obj_or_url=None, link_text='',link_class=''):
    context = post_to_telegram_url(context, title, obj_or_url)
    context['link_class'] = link_class 
    context['link_text'] = link_text or 'Post to Telegram'
    return context


@register.simple_tag(takes_context=True)
def post_to_whatsapp_url(context, obj_or_url=None):
    request = context['request']
    url = _build_url(request, obj_or_url)
    context['whatsapp_url'] = WHATSAPP_ENDPOINT % urlencode(url)
    return context


@register.inclusion_tag('django_social_share/templatetags/post_to_whatsapp.html', takes_context=True)
def post_to_whatsapp(context, obj_or_url=None, link_text='',link_class=''):
    context = post_to_whatsapp_url(context, obj_or_url)
    context['link_class'] = link_class 
    context['link_text'] = link_text or 'Post to WhatsApp'
    return context


@register.simple_tag(takes_context=True)
def save_to_pinterest_url(context, obj_or_url=None):
    request = context['request']
    url = _build_url(request, obj_or_url)
    context['pinterest_url'] = PINTEREST_ENDPOINT % urlencode(url)
    return context


@register.inclusion_tag('django_social_share/templatetags/save_to_pinterest.html', takes_context=True)
def save_to_pinterest(context, obj_or_url=None, pin_count=False, link_class=''):
    context = save_to_pinterest_url(context, obj_or_url)
    context['link_class'] = link_class 
    context['pin_count'] = pin_count
    return context


@register.inclusion_tag('django_social_share/templatetags/pinterest_script.html', takes_context=False)
def add_pinterest_script():
    pass

@register.simple_tag(takes_context=True)
def copy_to_clipboard_url(context, obj_or_url=None):
    request = context['request']
    url = _build_url(request, obj_or_url)
    context['copy_url'] = url
    return context

@register.inclusion_tag('django_social_share/templatetags/copy_to_clipboard.html', takes_context=True)
def copy_to_clipboard(context, obj_or_url, link_text='', link_class=''):
    context = copy_to_clipboard_url(context, obj_or_url)
    
    context['link_class'] = link_class
    context['link_text'] = link_text or 'Copy to clipboard'
    return context

@register.inclusion_tag('django_social_share/templatetags/copy_script.html', takes_context=False)
def add_copy_script():
    pass


from django.template import Context, Template
from django.test import TestCase, RequestFactory


class TemplateTagsTest(TestCase):

    def setUp(self):
        self.factory = RequestFactory()
        self.context = Context({
            'url': 'http://example.com',
            'text': 'example',
            'subject': 'Example Domain',
            'link_text':'',
            'link_class': 'example_class',
            'request': self.factory.get('/')
        })


    def test_twitter(self):
        template = Template("{% load social_share %} {% post_to_twitter text url %}")
        result = template.render(self.context)
        expected = ' <div class="tweet-this">\n    <a href="https://twitter.com/intent/tweet?text=example%20http%3A//example.com" class="meta-act-link meta-tweet " target="_blank">Post to Twitter</a>\n</div>\n'
        self.assertEqual(result, expected)

    def test_facebook(self):
        template = Template("{% load social_share %} {% post_to_facebook url text %}")
        result = template.render(self.context)
        expected = ' <div class="facebook-this">\n    <a href="https://www.facebook.com/sharer/sharer.php?u=http%3A//example.com" class="" target="_blank">example</a>\n</div>\n'
        self.assertEqual(result, expected)

    def test_gplus(self):
        template = Template("{% load social_share %} {% post_to_gplus url text %}")
        result = template.render(self.context)
        expected = ' <div class="gplus-this">\n    <a href="https://plus.google.com/share?url=http%3A//example.com" class="" target="_blank">example</a>\n</div>\n'
        self.assertEqual(result, expected)

    def test_mail_url(self):
        template = Template("{% load social_share %} {% send_email_url subject text url %}")
        template.render(self.context)
        expected = 'mailto:?subject=Example%20Domain&body=example%20http%3A//example.com'
        self.assertEqual(self.context['mailto_url'], expected)

    def test_mail(self):
        template = Template("{% load social_share %} {% send_email subject text url %}")
        result = template.render(self.context)
        expected = ' <div class="mail-this">\n    <a href="mailto:?subject=Example%20Domain&body=example%20http%3A//example.com" class="">Share via email</a>\n</div>\n'
        self.assertEqual(result, expected)

    def test_linkedin(self):
        template = Template("{% load social_share %} {% post_to_linkedin url %}")
        result = template.render(self.context)
        expected = ' <div class="linkedin-this ">\n  <script src="https://platform.linkedin.com/in.js" type="text/javascript">lang: en_US</script>\n  <script type="IN/Share" data-url="http://example.com"></script>\n</div>\n'
        self.assertEqual(result, expected)

    def test_reddit(self):
        template = Template("{% load social_share %} {% post_to_reddit text url text %}")
        result = template.render(self.context)
        expected = ' <div class="reddit-this">\n    <a href="https://www.reddit.com/submit?title=example&url=http%3A//example.com" class="" target="_blank">example</a>\n</div>\n'
        self.assertEqual(result, expected)

    def test_telegram(self):
        template = Template("{% load social_share %} {% post_to_telegram text url text %}")
        result = template.render(self.context)
        expected = ' <div class="telegram-this">\n    <a href="https://t.me/share/url?text=example&url=http%3A//example.com" class="" target="_blank">example</a>\n</div>\n'
        self.assertEqual(result, expected)

    def test_whatsapp(self):
        template = Template("{% load social_share %} {% post_to_whatsapp url text %}")
        result = template.render(self.context)
        expected = ' <div class="whatsapp-this">\n    <a href="https://api.whatsapp.com/send?text=http%3A//example.com" class="" target="_blank">example</a>\n</div>\n'
        self.assertEqual(result, expected)

    def test_pinterest(self):
        template = Template("{% load social_share %} {% save_to_pinterest url %}")
        result = template.render(self.context)
        expected = ' <div class="pinterest-this ">\n    <a data-pin-do="buttonPin" href="https://www.pinterest.com/pin/create/button/?url=http%3A//example.com" target="_blank"></a>\n</div>\n'
        self.assertEqual(result, expected)

    def test_pinterest_script(self):
        template = Template("{% load social_share %} {% add_pinterest_script %}")
        result = template.render(self.context)
        expected = ' <script async defer src="https://assets.pinterest.com/js/pinit.js"></script>\n'
        self.assertEqual(result, expected)

    def test_clipboard(self):
        template = Template("{% load social_share %} {% copy_to_clipboard url %}")
        result = template.render(self.context)
        expected = ' <div class="copy-this">\n    <button data-copy-btn="buttonCopy" data-copy-url="http://example.com" class="">Copy to clipboard</button>\n</div>\n'
        self.assertEqual(result, expected)

    def test_twitter_with_class(self):
        template = Template("{% load social_share %} {% post_to_twitter text url  link_text link_class %}")
        result = template.render(self.context)
        expected = ' <div class="tweet-this">\n    <a href="https://twitter.com/intent/tweet?text=example%20http%3A//example.com" class="meta-act-link meta-tweet example_class" target="_blank">Post to Twitter</a>\n</div>\n'
        self.assertEqual(result, expected)

    def test_facebook_with_class(self):
        template = Template("{% load social_share %} {% post_to_facebook url text link_class %}")
        result = template.render(self.context)
        expected = ' <div class="facebook-this">\n    <a href="https://www.facebook.com/sharer/sharer.php?u=http%3A//example.com" class="example_class" target="_blank">example</a>\n</div>\n'
        self.assertEqual(result, expected)

    def test_gplus_with_class(self):
        template = Template("{% load social_share %} {% post_to_gplus url text link_class %}")
        result = template.render(self.context)
        expected = ' <div class="gplus-this">\n    <a href="https://plus.google.com/share?url=http%3A//example.com" class="example_class" target="_blank">example</a>\n</div>\n'
        self.assertEqual(result, expected)

    def test_mail_with_class(self):
        template = Template("{% load social_share %} {% send_email subject text url link_text link_class %}")
        result = template.render(self.context)
        expected = ' <div class="mail-this">\n    <a href="mailto:?subject=Example%20Domain&body=example%20http%3A//example.com" class="example_class">Share via email</a>\n</div>\n'
        self.assertEqual(result, expected)

    def test_linkedin_with_class(self):
        template = Template("{% load social_share %} {% post_to_linkedin url link_class %}")
        result = template.render(self.context)
        expected = ' <div class="linkedin-this example_class">\n  <script src="https://platform.linkedin.com/in.js" type="text/javascript">lang: en_US</script>\n  <script type="IN/Share" data-url="http://example.com"></script>\n</div>\n'
        self.assertEqual(result, expected)

    def test_reddit_with_class(self):
        template = Template("{% load social_share %} {% post_to_reddit text url text link_class %}")
        result = template.render(self.context)
        expected = ' <div class="reddit-this">\n    <a href="https://www.reddit.com/submit?title=example&url=http%3A//example.com" class="example_class" target="_blank">example</a>\n</div>\n'
        self.assertEqual(result, expected)

    def test_telegram_with_class(self):
        template = Template("{% load social_share %} {% post_to_telegram text url text link_class %}")
        result = template.render(self.context)
        expected = ' <div class="telegram-this">\n    <a href="https://t.me/share/url?text=example&url=http%3A//example.com" class="example_class" target="_blank">example</a>\n</div>\n'
        self.assertEqual(result, expected)

    def test_whatsapp_with_class(self):
        template = Template("{% load social_share %} {% post_to_whatsapp url text link_class %}")
        result = template.render(self.context)
        expected = ' <div class="whatsapp-this">\n    <a href="https://api.whatsapp.com/send?text=http%3A//example.com" class="example_class" target="_blank">example</a>\n</div>\n'
        self.assertEqual(result, expected)

    def test_pinterest_with_class(self):
        template = Template("{% load social_share %} {% save_to_pinterest url  False link_class %}")
        result = template.render(self.context)
        expected = ' <div class="pinterest-this example_class">\n    <a data-pin-do="buttonPin" href="https://www.pinterest.com/pin/create/button/?url=http%3A//example.com" target="_blank"></a>\n</div>\n'
        self.assertEqual(result, expected)

    def test_clipboard_with_class(self):
        template = Template("{% load social_share %} {% copy_to_clipboard url text link_class %}")
        result = template.render(self.context)
        expected = ' <div class="copy-this">\n    <button data-copy-btn="buttonCopy" data-copy-url="http://example.com" class="example_class">example</button>\n</div>\n'
        self.assertEqual(result, expected)


import io
import os
from setuptools import setup, find_packages


here = os.path.abspath(os.path.dirname(__file__))

with io.open(os.path.join(here, 'README.rst'), encoding='utf-8') as fp:
    README = fp.read()

with open(os.path.join(here, 'VERSION')) as version_file:
    version = version_file.read().strip()

requirements = []
test_requirements = [
    "Django>=2.0",
]

setup(
    name="django-social-share",
    version=version,
    description="Templatetags for 'tweet this' and 'share on facebook'",
    long_description=README,
    url='https://github.com/fcurella/django-social-share',
    license='MIT',
    author='Flavio Curella',
    author_email='flavio.curella@curella.org',
    packages=find_packages(exclude=['tests']),
    include_package_data=True,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: Information Technology',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Django',
    ],
    install_requires=requirements,
    tests_require=test_requirements,
    test_suite='django_social_share.tests.runtests.runtests'
)
